﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;

namespace howto_row_column_montage
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private Padding Spacing = new Padding(0);

        // Initialize the background color menu item.
        private void Form1_Load(object sender, EventArgs e)
        {
            // Use light colors plus the initial background
            // color for the dialog's custom colors.
            int bg_color =
                flpImages.BackColor.B * 256 * 256 +
                flpImages.BackColor.G * 256 +
                flpImages.BackColor.R;
            int[] bg_colors = {
                bg_color, 0xFFC0C0, 0xFFE0C0, 0xFFFFC0, 0xC0FFC0,
                0xC0FFFF, 0xC0C0FF, 0xFFC0FF, 0xE0E0E0, 0xFF8080,
                0xFFC080, 0xFFFF80, 0x80FF80, 0x80FFFF, 0x8080FF,
                0xFF80FF
            };
            cdBackground.CustomColors = bg_colors;

            // Display the current color on the menu item.
            SetBgColor(flpImages.BackColor);
        }

        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            if (ofdImages.ShowDialog() == DialogResult.OK)
            {
                foreach (string filename in ofdImages.FileNames)
                {
                    PictureBox pic = new PictureBox();
                    pic.SizeMode = PictureBoxSizeMode.AutoSize;
                    pic.Margin = Spacing;
                    pic.Image = LoadBitmapUnlocked(filename);
                    pic.Parent = flpImages;
                }
            }
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdImages.ShowDialog() == DialogResult.OK)
            {
                // Find the maximum X and Y bounds.
                int max_x = 0;
                int max_y = 0;
                foreach (PictureBox pic in flpImages.Controls)
                {
                    if (max_x < pic.Right) max_x = pic.Right;
                    if (max_y < pic.Bottom) max_y = pic.Bottom;
                }
                max_x += Spacing.Right;
                max_y += Spacing.Bottom;

                // Create and save the montage.
                Bitmap bm = new Bitmap(max_x, max_y);
                using (Graphics gr = Graphics.FromImage(bm))
                {
                    gr.Clear(flpImages.BackColor);

                    // Find the maximum X and Y bounds.
                    foreach (PictureBox pic in flpImages.Controls)
                    {
                        gr.DrawImage(pic.Image,
                            pic.Left, pic.Top,
                            pic.Width, pic.Height);
                    }
                }
                SaveImage(bm, sfdImages.FileName);
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        private void mnuSettingsSpacing_Click(object sender, EventArgs e)
        {
            DlgSpacing dlg = new DlgSpacing();
            dlg.Spacing = Spacing;

            if (dlg.ShowDialog() == DialogResult.OK)
            {
                Spacing = dlg.Spacing;
                foreach (Control ctl in flpImages.Controls)
                    ctl.Margin = Spacing;
            }
        }

        private void mnuSettingsBgColor_Click(object sender, EventArgs e)
        {
            cdBackground.Color = flpImages.BackColor;
            if (cdBackground.ShowDialog() == DialogResult.OK)
            {
                SetBgColor(cdBackground.Color);
            }
        }

        private void SetBgColor(Color color)
        {
            const int wid = 24;

            flpImages.BackColor = color;
            Bitmap bm = new Bitmap(wid, wid);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.Clear(color);
                gr.DrawRectangle(Pens.Black, 0, 0, wid - 1, wid - 1);
            }
            mnuSettingsBgColor.Image = bm;
        }

        private void mnuFileReset_Click(object sender, EventArgs e)
        {
            flpImages.Controls.Clear();
        }

        // Set flow direction.
        private void mnuFlow_Click(object sender, EventArgs e)
        {
            // Uncheck all menu items.
            ToolStripMenuItem[] items =
            {
                mnuFlowLeftToRight,
                mnuFlowRightToLeft,
                mnuFlowTopToBottom,
                mnuFlowBottomToTop,
            };
            foreach (ToolStripMenuItem item in items)
            {
                item.Checked = false;
            }

            // Check the correct item.
            ToolStripMenuItem selected = sender as ToolStripMenuItem;
            selected.Checked = true;

            // Update the FlowLayoutPanel.
            switch (selected.Text)
            {
                case "&Left to Right":
                    flpImages.FlowDirection = FlowDirection.LeftToRight;
                    break;
                case "&Right to Left":
                    flpImages.FlowDirection = FlowDirection.RightToLeft;
                    break;
                case "&Top to Bottom":
                    flpImages.FlowDirection = FlowDirection.TopDown;
                    break;
                case "&Bottom to Top":
                    flpImages.FlowDirection = FlowDirection.BottomUp;
                    break;
            }
        }
    }
}
